export install="/usr/bin/install -c"
export bash="/usr/bin/bash"
export bison="/opt/cfs/conda-bld/crosstool-ng_1617226458206/_build_env/bin/bison"
export awk="/usr/bin/env gawk"
export grep="/usr/bin/env grep"
export gmake="/usr/bin/make"
export gnumake="/usr/bin/make"
export make="/usr/bin/make"
export python="/opt/cfs/bin/python3"
export sed="/usr/bin/env sed"
export libtool="/opt/cfs/conda-bld/crosstool-ng_1617226458206/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_pl/bin/libtool"
export libtoolize="/opt/cfs/conda-bld/crosstool-ng_1617226458206/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_pl/bin/libtoolize"
export objcopy="/usr/bin/env objcopy"
export objdump="/usr/bin/env objdump"
export readelf="/usr/bin/env readelf"
export patch="/usr/bin/env patch"
export md5sum="/usr/bin/md5sum"
export sha1sum="/usr/bin/sha1sum"
export sha256sum="/usr/bin/sha256sum"
export sha512sum="/usr/bin/sha512sum"
